/**
 * This class is the main class of the game. It handles the game loop containing the game logic
 * and initiates the game.
 * 
 * @author David Laurell <david.laurell@gmail.com>
 * 
 * Copyright 2008 David Laurell
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package game;

import java.util.ArrayList;

import gameobjects.Player;
import gameobjects.Asteroid;
import gameobjects.Shot;

import org.newdawn.slick.AppGameContainer;
import org.newdawn.slick.BasicGame;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.InputListener;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.svg.InkscapeLoader;
import org.newdawn.slick.svg.SimpleDiagramRenderer;
import javax.swing.JOptionPane;

public class Game extends BasicGame implements InputListener {
	ArrayList<Asteroid> asteroids;
	ArrayList<Shot> shots;
	Player player;
	Highscore highscore;
	
	Input input;
	
	SimpleDiagramRenderer imgShip, imgShield, imgAsteroid, imgAsteroidMedium, imgAsteroidSmall, imgShot;
	SimpleDiagramRenderer imgBackground;
	Color bgcolor;
	
	int lvl, counter, multiplier = 0, shotsFired=0, hits=0;

	public Game()
	{
		//The title of the game
		super("Daveroids 0.5!");
		highscore = new Highscore();
	}

	public void init(GameContainer container) throws SlickException
	{
		input = container.getInput();
		
		/* Loads all the vector graphics */
		try {
			imgAsteroid = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/asteroid.svg"));
			imgAsteroidMedium = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/asteroid_medium.svg"));
			imgAsteroidSmall = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/asteroid_small.svg"));
			
			imgShot = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/shot.svg"));
			imgShip = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/ship.svg"));
			imgShield = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/shield.svg"));
			imgBackground = new SimpleDiagramRenderer(InkscapeLoader.load("pictures/background.svg"));
			
			player = new Player(400,300,imgShip,imgShield);
		} catch (SlickException e) {
			e.printStackTrace();
		}
		
		asteroids = new ArrayList<Asteroid>();
		shots = new ArrayList<Shot>();
		highscore = new Highscore();
		
		levelHandler();
		randomBG();
	}	

	public void randomBG() {
		bgcolor = new Color((int)(Math.random()*128+64),(int)(Math.random()*128+64),(int)(Math.random()*192+64));
	}
	/**
	 * Convenience method for checking if a key is pressed
	 * @param keyCode
	 * @return boolean
	 */
	public boolean isKeyPressed(int keyCode)
	{
		return (input.isKeyPressed(keyCode) || input.isKeyDown(keyCode));
	}
	/**
	 * This method draws a frame
	 */
	public void render(GameContainer container, Graphics g) throws SlickException {
		g.setBackground(bgcolor);
		
		imgBackground.render(g);

		for(Asteroid a : asteroids)
			a.draw(g);

		for(Shot s : shots)
			s.draw(g);
		
		if(player.getLives() > 0)
			player.draw(g);
		else {
			g.setColor(Color.white);
			g.drawString("GAME OVER", 350, 150);
			
			int i = 0;
			for(HighscoreItem hi:highscore.getList()) {
				if(hi != null) {
					g.drawString(hi.getName(), 300, 190+i*20);
					g.drawString(""+hi.getScore(), 450, 190+i*20);
					i++;
				}
			}
			i++;
			g.drawString("Press F2 to start a new game", 300, 190+i*20);
			g.resetTransform();
		}
		
		for(int i=0;i<player.getLives();i++) {
			g.translate((float)746 - i * 32,(float)546);
			imgShield.render(g);
			g.resetTransform();
		}

		g.setColor(Color.white);
		
		String accuracyString = "Accuracy: ";
		if(shotsFired > 0 && hits > 0)
			accuracyString += "" + (int)(((float)hits / (float)shotsFired) * 100);
		else
			accuracyString += "0";
		
		g.drawString(accuracyString + "%", 10, 10);
			
		String points = "Points: " + player.getPoints();
		if(multiplier > 1)
			points += " x" + multiplier;
		
		g.drawString(points, 625, 10);
		g.drawString("Level " + lvl, 375, 10);
	}
	
	/**
	 * Handle game logic, like moving ;)
	 */
	public void update(GameContainer container, int delta) throws SlickException {
		handleControls(container, delta);
		if(!player.isDead()) {
			player.move(delta);

			if(player.makeShot()) {
				Shot s = new Shot(player,imgShot);
				shots.add(s);
				shotsFired++;
			}
		}
		
		player.update(delta);
		
		for(Asteroid a : asteroids)
			a.move(delta);
		
		for(Shot s : shots)
			s.move(delta);
		
		checkCollisions();
		levelHandler();
	}
	
	public void startOver() {
		counter = 0;
		lvl = 0;
		multiplier = 0;
		hits = 0;
		shotsFired = 0;
		player = new Player(400,300,imgShip,imgShield);
		asteroids = new ArrayList<Asteroid>();
		shots = new ArrayList<Shot>();
		levelHandler();
	}
	
	public void levelHandler() {
		if(counter == 0) {
			lvl++;
			addAsteroids(lvl+2);
			randomBG();
			if(lvl > 1 && hits > 0 && shotsFired > 0)
				player.addPoints((int)(500 * ((float)hits/shotsFired)));
		}
	}
	
	public void checkCollisions()
	{	
		if(!player.isDead()) {
			for(int i=0;i<asteroids.size();i++) {
				if(player.isCollision(asteroids.get(i))) {
					Asteroid a = asteroids.get(i);
					createNewAsteroids(a);
					
					if(!player.isInvulnerable()) {
						player.setDead();
						if(player.getLives() == 0 && player.isDead()) {
							if(highscore.isHighscore(player.getPoints())) {
								try {
									String name = JOptionPane.showInputDialog("Highscore! Input your name!");
									player.setName(name);
								}catch(Exception e) {}
								
								highscore.add(player);
							}
						}
					}
					counter--;
					multiplier = 0;
					
					asteroids.remove(i);
					break;
				}
			}
		}

		for(Shot s: shots) {
			boolean remove = false;
			
			for(Asteroid a : asteroids) {
				if(s.isCollision(a)) {
					counter--;
					if(multiplier < 5)
						multiplier++;
					
					hits++;
					
					player.addPoints(a.getPoints() * (multiplier+1));
					
					asteroids.remove(a);
					remove = true;
					createNewAsteroids(a);
					break;
				}
			}
			if(s.created + 750 < System.currentTimeMillis() || remove) {
				if(!remove)
					multiplier = 0;

				shots.remove(s);
				break;
			}
		}
	}
	
	public void createNewAsteroids(Asteroid asteroid) {
		if(asteroid.size == Asteroid.MEDIUM) {
			for (int i = 0; i < 3; i++) {
				Asteroid a1 = new Asteroid(asteroid.getX(),asteroid.getY(), imgAsteroidSmall, Asteroid.SMALL, lvl);
				a1.setDirection((float) asteroid.getDirection() + i * 120);
				asteroids.add(a1);				
			}
		}
		else if(asteroid.size == Asteroid.LARGE) {
			for(int i=0;i<2;i++) {
				Asteroid a1 = new Asteroid(asteroid.getX(),asteroid.getY(), imgAsteroidMedium, Asteroid.MEDIUM, lvl);
				a1.setDirection((float) asteroid.getDirection() + i * 120 - 60);
				asteroids.add(a1);
			}
		}
	}
	
	public void handleControls(GameContainer container, int delta)
	{		
		boolean leftPressed = input.isKeyDown(Input.KEY_LEFT);
		boolean rightPressed = input.isKeyDown(Input.KEY_RIGHT);
		boolean upPressed = input.isKeyDown(Input.KEY_UP);
		boolean downPressed = input.isKeyDown(Input.KEY_DOWN);
		
		if(input.isKeyDown(Input.KEY_ESCAPE))
			System.exit(0);
		else if(input.isKeyPressed(Input.KEY_F3)) {
			if(container.isPaused())
				container.resume();
			else
				container.pause();
		}
		else if(input.isKeyPressed(Input.KEY_F2)) {
			startOver();
			container.resume();
			input.clearKeyPressedRecord();
		}

		if(!container.isPaused() && !player.isDead()) {
			if(leftPressed && !rightPressed) {
				player.turnLeft();
			}
			else if(rightPressed && !leftPressed) {
				player.turnRight();
				}
				
			if(upPressed && !downPressed) {
				player.setSpeed(0.25f * delta);
			}
			else if(downPressed && !upPressed) {
				player.setSpeed(-0.2f * delta);
			}
			
			if(input.isKeyDown(Input.KEY_SPACE)) {
				player.shoot();
			}
		}
		
		input.clearKeyPressedRecord();
	}
	
	public void addAsteroids(int amount)
	{
		for(int i = 0; i < amount; i++)
		{
			boolean xy = Math.random() > 0.5;
			int x,y;
			
			if(xy) {
				x = (int)(Math.random() * 800);
				y = (int)Math.random() * 600;
			}
			else
			{
				x = (int)Math.random() * 800;
				y = (int)(Math.random() * 600);				
			}
			
			Asteroid asteroid = new Asteroid(x,y,imgAsteroid, Asteroid.LARGE, lvl);
			asteroids.add(asteroid);
			counter += 9;
		}
	}

	public void windowClosed() {
		
	}
	/**
	 * The main method just creates a new game object
	 * @param args
	 */
	public static void main(String[] args)
	{
		try{
			AppGameContainer container = new AppGameContainer(new Game());
			container.setDisplayMode(800,600, false);
			container.setTargetFrameRate(60);
			container.setShowFPS(false);
			container.getAlwaysRender();
			container.start();
		}
		catch(SlickException e){
			e.printStackTrace();
		}
		
	}
	
}
